#!/usr/bin/perl
use strict;

# This perl program parses an RTP dump file and validates that RTPEvent 
# packets received are in accordance to the RFC 4733. It checks for the 
# following conditions in the RTPDUmp file, all these conditions must be
# true for a valid RTPDUMP file. Below are the conditions
# 1. All DTMFs should arrive in the right order
# 2. For each DTMF, there should be atleast one packet with E bit set
# 3. Sequence number should be incremented even for the retransmissions of 
#    end packets (End bit set). Refer to RFC 4733 section 2.5.1.6
# 4. Sequence number of RTP Event packets should be in sync with seq number
#    for regular AUDIO packets
# 5. One DTMF should finish before the other one starts
# 6. There should be a gap of atleast 40ms between the two DTMFs. See the 
#    section 3.1, RFC 4733 and the difference between start of two DTMFs
#    has to be more than (duration of previous DTMF + 40 ms)
# 7. SSRC for rtpevent packets should be in sync with SSRC for regular audio
#    packets
# 8. All RTP event packets for a DTMF should have the same timestamp
#
#  Note, this validator assumes that there will be no long duration DTMFs
# This perl program expects minium of three arguments on command line,
# First argument is the name of RTP dump, second argument is the payload
# type of the telephone-eventi, third argument and so on are the dtmfs
# dgitis expected in the order of their arrival
#                 
# Check the number of command lines arguments

my $num_args = @ARGV;
if ($num_args < 3)
{
	print "[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Insufficient arguments\n";
	exit(99);
}
else
{
	my $FILE_PTR;
	my $dump_file = $ARGV[0];
	my $dyn_payload_rtpevent = $ARGV[1]; #Payload number for RTPEvent packets
	my @exp_dtmfs = @ARGV[2..$num_args]; #DTMFs expected and order in which they are received
	my $exp_num_dtmfs = $num_args - 2; # Number of DTMs expected 
	if (open(FILE_PTR,$dump_file))
	{
		my $count=0; #Counter to iterate over the array
		#containing all the lines in the dump file
		my $line;  #String holding the current in dump file
		my @lines; #Array holidng all the lines in dump file
		my $dump_length=0; #Total number of lines in dump file

		my $dtmfs_processed = 0; # IT is an integer recording the
		#count of number of DTMFs processed
		#from the dump file
		my $curr_dtmf = '?';
		my $start_timestamp_curr_dtmf = 0; # It stores the timestamp for a new DTMF i.e RTPEvent 
		# packet with MARKET BIT SET TO TRUE
		my $duration_curr_dtmf = 0; #It stores the duration of DTMF received in RTPEvent packet
		my $end_bit_rec_curr_dtmf = 0; # It indicates that end packet has been received for a DTMF
		my $bad_packet = 0;
		my $rtp_seq_num = -1;
		my $rtp_ssrc = '?';
		my $curr_dtmf_timestamp = -1;
		my $retrans= 0;
		my $dtmf_begin_duration=0; #Duration contained in the DTMF begin packet
		my $curr_ts = 0;
		my $previous_ts = 0;

		# Read the dump file into arryt @lines 
		@lines = <FILE_PTR>;
		close FILE_PTR;
		$dump_length = @lines;

		# Process the dump file 
		for ($count=0;$count<$dump_length;$count++)
		{
			$line = @lines[$count];
			#Check for the payload type
			if ($line =~ /.*m=([01]).*pt=([0-9]+).*seq=([0-9]+).*ts=([0-9]+).*ssrc=0x(.{1,8})\ /)	
			{
				my $marker = $1;
				my $pt = $2;
				my $seq = $3;
				my $ssrc = $5;

				$previous_ts = $curr_ts;
				$curr_ts = $4; 

				#Very first thing to be done is the validation of SEQUENCE number and SSRC
				#SSRC allowed to change when marker bit is set,
				#but we want DTMF to always have the ssrc as the audio before it
				if (($rtp_seq_num == -1) && ($rtp_ssrc eq '?') 
					|| ($marker == 1 && $dyn_payload_rtpevent ne $pt))
				{
					#It is the first RTP packet
					$rtp_seq_num = $seq; # Set the RTP seq number for RTP stream	
					$rtp_ssrc = $ssrc; # SEt the SSRC for RTP Stream
				}
				elsif ($ssrc ne $rtp_ssrc)
				{
					print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Bad RTP packet,BAD SSRC received\n");
					print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Error detected for packet [$line]\n");
					$bad_packet = 1;
					last;

				}
				elsif ($curr_ts < $previous_ts)
				{
					
					print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Bad RTP packet,Decreasing packet timestamp, previous packet had timestamp [$previous_ts]\n");
					print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Error detected for packet [$line]\n");
					$bad_packet = 1;
					last;
				}
				elsif ($seq < $rtp_seq_num)
				{
					if ($seq == 0 && $rtp_seq_num == 65535)
					{
						#RTP sequence number rolled back
						$rtp_seq_num = $seq; # Update the current sequence number	
					}
					else
					{
						print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Bad RTP packet,BAD Sequence number [$seq]\n");
						print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Error detected for packet [$line]\n");
						$bad_packet = 1;
						last;
					}
				}
				elsif ($seq == $rtp_seq_num)
				{
					#On the loopback interface, there will be no retransmission except for the telephone
					#event packets with END bit set
					#RETRANS_FLAG will be reset later on, if it turns out to be the end packet for a DTMF
					print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Possible retransmission [$seq]\n");
					$retrans = 1;
				}
				else
				{
					$rtp_seq_num = $seq; # Update the current sequence number	

				}




				if ($dyn_payload_rtpevent eq $pt) # It is an RTPEvent packet
				{
					if ($line =~ /([0-9]+\.[0-9]+)\ .*m=([01]).*ts=([0-9]+).*dtmf=(.*)\ duration=([0-9]+)ms\ end=([01])/)	
					{
						my $timestamp = convertToMilliSec($1);
						my $marker = $2;
						my $rtp_ts = $3;
						my $dtmf = $4;
						my $duration = $5;
						my $end = $6;

						if ($marker == 1)
						{
							#It is a new DTMF

							if ($dtmfs_processed == $exp_num_dtmfs)
							{
								print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Extra DTMF received [$dtmf]\n");
								print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Expected DTMFs [$exp_num_dtmfs] DTMFs already processed [$dtmfs_processed]\n");
								print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Error detected for packet [$line]\n");
								$bad_packet = 1;
								last;
							}
							if ($curr_dtmf eq '?') # It implies that it is the first DTMF received
							{
								if ($dtmf ne $exp_dtmfs[$dtmfs_processed]) # Check if it was expected DTMF
								{	
									print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"] RECEIVED DTMF[$dtmf] EXPECTED DTMF [$exp_dtmfs[$dtmfs_processed]]\n");
									#Out of DTMF DTMF received, stop the processing
									print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]OUT OF ORDER DTMF RECEIVED\n");
									print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Error detected for packet [$line]\n");
									$bad_packet = 1;
									last;#Stop processing the dump files and exit with failure
								}
								$curr_dtmf = $dtmf; #Set it to be the current DTMF
								# It is a valid DTMF
								# Update the start timestamp for new DTMF
								$start_timestamp_curr_dtmf = $timestamp;
								$duration_curr_dtmf = $duration;

								#Set it as current DTMF
								$curr_dtmf = $dtmf;
								$end_bit_rec_curr_dtmf = 0;
								#Save the RTP TIMESTAMP for current timestamp
								$curr_dtmf_timestamp = $rtp_ts;

								$dtmf_begin_duration=$duration_curr_dtmf; #Save the duration from the BEGIN Packet

								#	print (__LINE__,":START timestamp FIRST ONE [$start_timestamp_curr_dtmf]\n",);


							}
							else
							{
								#	print (__LINE__,":TIMESTAMP NEW DTMF [$timestamp]\n");
								#This is not the first DTMF,so validate the conditions

								# Conditon 1. If the previous DTMF has finished
								if ($end_bit_rec_curr_dtmf ==  0)
								{
									print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]New DTMF received before end packet of the current DTMF\n");
									print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Error detected for packet [$line]\n");
									$bad_packet = 1;
									last; #New DTMF received before the end packet was received
									#for the curr DTMF
								}
								elsif ($dtmf ne $exp_dtmfs[$dtmfs_processed]) # Check if it was expected DTMF
								{	
									print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"] RECEIVED DTMF[$dtmf] EXPECTED DTMF [$exp_dtmfs[$dtmfs_processed]]\n");
									#Out of DTMF DTMF received, stop the processing
									print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]OUT OF ORDER DTMF RECEIVED\n");
									print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Error detected for packet [$line]\n");
									$bad_packet = 1;
									last;#Stop processing the dump files and exit with failure
								}
								else  # Condition If there is enough gap after the two DTMFs
								{
									my $gap_btw_dtmfs = ($timestamp - ($start_timestamp_curr_dtmf + $duration_curr_dtmf- $dtmf_begin_duration) );
									if( $gap_btw_dtmfs < 40)
									{
										print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Not enough gap between two DTMFs,DTMFs received at a gap of [$gap_btw_dtmfs]\n");
										print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Error detected for packet [$line]\n");
										$bad_packet = 1;
										last;
									}
									else
									{
										# It is a valid DTMF
										# Update the start timestamp for new DTMF
										$start_timestamp_curr_dtmf = $timestamp;
										$duration_curr_dtmf = $duration;

										#Set it as current DTMF
										$curr_dtmf = $dtmf;
										$end_bit_rec_curr_dtmf = 0;
										#Save the RTP TIMESTAMP for current timestamp
										$curr_dtmf_timestamp = $rtp_ts;
										$dtmf_begin_duration=$duration_curr_dtmf; #Save the duration from the BEGIN Packet

									}
								}

							}
						}
						elsif ($end == 1) # End packet received for curr DTMF
						{
							if ($dtmf eq $curr_dtmf)
							{
								#Validate if this RTP packet has the same DTMF
								#as previous packets for this DTMF
								if ($curr_dtmf_timestamp != $rtp_ts)
								{
									print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Inconsistant timestamp received for a DTMF\n");
									print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Error detected for packet [$line]\n");
									$bad_packet = 1;
									last;
								}

								#Update the duration of current DTMF
								$duration_curr_dtmf = $duration;
								#End packet received for the current DTMF
								$duration_curr_dtmf = $duration;

								#The IF condition is required, as there could be multiple
								#END packets received for one DTMF, but we will increment 
								#the variable $dtmfs_processed only once
								if ($end_bit_rec_curr_dtmf == 0)
								{
									$end_bit_rec_curr_dtmf = 1;
									$dtmfs_processed += 1; # Increment the number
									# of DTMFs processed
								}
								# Check if there was a retransmission
								if ($retrans == 1)
								{
									$retrans = 0; #Reset the retransmission as it
									              #is allowed for end packets of DTMF
								}
								#print (__LINE__,":DURATION [$duration_curr_dtmf]\n");
							}
							else
							{
								print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Unexpected RTPEvent packet received with END set to 1, while some other DTMF was playing\n");
								$bad_packet = 1;
								last;
							}

						}
						else
						{
							#It should be an udpate for current DTMF 
							#To be sure verify that DTMF received is
							# the current DTMF
							if ($dtmf eq $curr_dtmf)
							{
								#Validate if this RTP packet has the same DTMF
								#as previous packets for this DTMF
								if ($curr_dtmf_timestamp != $rtp_ts)
								{
									print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Inconsistant timestamp received for a DTMF\n");
									print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Error detected for packet [$line]\n");
									$bad_packet = 1;
									last;
								}
								#Update the duration of current DTMF
								$duration_curr_dtmf = $duration;
								#	print (__LINE__,":DURATION [$duration_curr_dtmf]\n");
							}
							else
							{
								print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]New DTMF received with MARKER set as false\n");
								print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Error detected for packet [$line]\n");
								$bad_packet = 1;
								last;

							}

						}

					} #End of IF checking for the necessary parametres in RTP Event packets
					else
					{
						print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Invalid RTPEvent packet received\n");
						print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Error detected for packet [$line]\n");
						$bad_packet = 1;
						last;#Stop processing the dump files and exit with failure
					}



				} #End of IF checking for RTPEVENT packets

				if ($retrans == 1)
					
				{
					print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Duplicate Sequence numbers detected\n");
					print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Error detected for packet [$line]\n");
					$bad_packet = 1;
					last;#Stop processing the dump files and exit with failure
				}

			}

		} # End of for loop processing the dump file


		if ($bad_packet eq 1)
		{
			exit(99);
		}
		else
		{
			print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Number of DTMFs Processed [$dtmfs_processed]\n");
			if ($dtmfs_processed == $exp_num_dtmfs)
			{
				exit(0);
			}
			else
			{
				print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Unexpected number of DTMFs received\n");
				print ("[RFC4733_DTMF_VALIDATOR:",__LINE__,"]Expected DTMFs [$exp_num_dtmfs] DTMFs processed [$dtmfs_processed]\n");
				exit(99);
			}
		}
	}
	else
	{
		print "[RFC4733_DTMF_VALIDATOR:",__LINE__,"] Unable to open the dump file [$dump_file]\n";
		exit(99);
	}

}

# This subroutine converts the timestamp in RTPDUMP file into milliseconds
sub convertToMilliSec() 
{
	my ($timestamp) = @_;
	#	print ("DUMP TIMESTAMP [$timestamp]");
	my(@arr) = split(/\./,$timestamp);
	#	print ("DUMP TIMESTAMP IN MILLISECONDS[$arr[0]] [$arr[1]]");
	my($msec) = ($arr[0]*1000) + ($arr[1]/1000);
	return $msec;

}
